<?php

namespace Modules\Sd\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Pp\Entities\CustomerOrder;
use Modules\Sd\Entities\Shipment;

class ShipmentController extends Controller
{

    // save

    public function saveSchedule(Request $request)
    {
        try {
            $vesselSchedule = new Shipment();
            $vesselSchedule->user_id = Auth::user()->id;
            $vesselSchedule->shipment_no = $request->get('shipment_no');
            $vesselSchedule->shipment_type = $request->get('shipment_type');
            $vesselSchedule->loading_date = $request->get('loading_date');
            $vesselSchedule->customer_order_id = $request->get('customer_order_id');
            $vesselSchedule->freight_by = $request->get('freight_by');
            $vesselSchedule->no_of_container = $request->get('no_of_container');
            $vesselSchedule->container_type = $request->get('container_type');
            $vesselSchedule->status = $request->get('status');

            if ($vesselSchedule->save()) {
                $responseBody = $this->responseBody(true, "ShipmentController", "saveSchedule", "Success");
            }


        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ShipmentController", "saveSchedule", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function update(Request $request, $id)
    {
        try {
            $shipment = Shipment::find($id);
            $shipment->user_id = Auth::user()->id;
            $shipment->shipment_no = $request->get('shipment_no');
            $shipment->shipment_type = $request->get('shipment_type');
            $shipment->loading_date = $request->get('loading_date');
            $shipment->customer_order_id = $request->get('customer_order_id');
            $shipment->freight_by = $request->get('freight_by');
            $shipment->no_of_container = $request->get('no_of_container');
            $shipment->container_type = $request->get('container_type');
            $shipment->status = $request->get('status');

            if ($shipment->update()) {

                $responseBody = $this->responseBody(true, "shipmentController", "updated", "success");
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "shipmentController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }
    public function deleteShipmentData($id)
    {
        try {
            $shipment = Shipment::where('shipment_id', '=', $id)->get();
            foreach ($shipment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "ShipmentController", "deleteShipmentData", "success");
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ShipmentController", "deleteShipmentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function allShipment()
    {
        try {
            $query = 'SELECT
            sd_shipment.*,
            pp_customer_orders.factory_po_num AS factory_order_no
            FROM
            pp_customer_orders
            INNER JOIN
            sd_shipment
            ON
            pp_customer_orders.order_id = sd_shipment.customer_order_id
            ORDER BY
            sd_shipment.created_at DESC';
            $shipmnets = DB::select($query);
            $result = array();

            foreach ($shipmnets as $shipment) {
                array_push($result, [
                    "shipment_id" => $shipment->shipment_id,
                    "shipment_no" => $shipment->shipment_no,
                    "shipment_type" => $shipment->shipment_type,
                    "loading_date" => $shipment->loading_date,
                    "customer_order_id" => $shipment->customer_order_id,
                    "factory_order_no" => $shipment->factory_order_no,
                    "freight_by" => $shipment->freight_by,
                    "no_of_container" => $shipment->no_of_container,
                    "container_type" => $shipment->container_type,
                    "status" => $shipment->status,
                ]);
            }
            $responseBody = $this->responseBody(true, "ShipmentController", "getAllShipment", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ShipmentController", "getAllShipment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getShipmentData(Request $request, $id)
    {
        try {
            $shipment = Shipment::find($id);

            $responseBody = $this->responseBody(true, "shipmentController", "getShipmentData", $shipment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "shipmentController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function genarateExternalID()
    {
        $id = 0;
        $result = DB::select('SELECT MAX(shipment_id) AS id FROM sd_shipment')[0];
        if ($result) {
            $id = $result->id;
        }
        return response()->json(["shipment_id" => ($id += 1)]);

    }

    public function getOrderNo()
    {
        try {

            $customer_orders = CustomerOrder::orderBy('order_id', 'DESC')->get();

            $collection = [];

            for ($i = 0; $i < count($customer_orders); $i++) {
                array_push($collection, ["img" => "", "id" => $customer_orders[$i]['order_id'], "value" => $customer_orders[$i]['factory_po_num'] . " - " . $customer_orders[$i]['customer_name']]);
            }
            $responseBody = $this->responseBody(true, "ShipmentController", "getCustomerOrderNo", $collection);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ShipmentController", "getCustomerOrderNo", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    public function vsCheckAvailability($id)
    {

        try {
            $query = "SELECT
            sd_shipment.customer_order_id,
            pp_customer_orders.factory_po_num
        FROM
            pp_customer_orders
            INNER JOIN sd_shipment ON pp_customer_orders.order_id = sd_shipment.customer_order_id
            INNER JOIN sd_vessel_schedules ON pp_customer_orders.order_id = sd_vessel_schedules.order_id
        WHERE
            sd_shipment.shipment_id = {$id}";

            $vessel_availability = DB::select($query);

            $responseBody = $this->responseBody(true, "ShipmentController", "getNcrData", $vessel_availability);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ShipmentController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function roCheckAvailability($id)
    {

        try {
            $query = "SELECT
            sd_shipment.customer_order_id,
            sd_release_order.factory_po
        FROM
            sd_release_order
            INNER JOIN sd_shipment ON sd_release_order.shipment_id = sd_shipment.shipment_id
        WHERE
            sd_shipment.shipment_id = {$id}";

            $release_order_availability = DB::select($query);

            $responseBody = $this->responseBody(true, "ShipmentController", "getNcrData", $release_order_availability);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ShipmentController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }
    public function biCheckAvailability($id)
    {

        try {
            $query = "SELECT
            sd_shipment.customer_order_id,
            sd_bl_instruction.factory_po
        FROM
            sd_bl_instruction
            INNER JOIN sd_shipment ON sd_bl_instruction.shipment_id = sd_shipment.shipment_id
        WHERE
            sd_shipment.shipment_id = {$id}";

            $bi_instruction_availability = DB::select($query);

            $responseBody = $this->responseBody(true, "ShipmentController", "getBlInstruction", $bi_instruction_availability);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ShipmentController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }
    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
